@@ -18,7 +18,7 @@
 #include <haproxy/signal-t.h>
 
 void mworker_proc_list_to_env();
-void mworker_env_to_proc_list();
+int mworker_env_to_proc_list();
 
 
 void mworker_block_signals();
@@ -40,5 +40,6 @@ int mworker_ext_launch_all();
 void mworker_kill_max_reloads(int sig);
 
 void mworker_free_child(struct mworker_proc *);
+void mworker_cleanup_proc();
 
 #endif /* _HAPROXY_MWORKER_H_ */
